<?php

namespace EventPortal\Tests\Unit\Delegate;

use Carbon\Carbon;
use EventPortal\Domain\Delegate\DelegateDashboard;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class DelegateDashboardTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function gets_dashboard_data()
    {
        Carbon::setTestNow(Carbon::parse('08 September 2020'));
        $delegate = factory(Delegate::class)->create();

        $upcoming = factory(Event::class)->create([
            'name' => 'Upcoming event',
            'slug' => 'upcoming-event',
            'status' => 'open',
        ]);
        $upcoming->agendas()->create([
            'date' => now()->addDay(),
        ]);
        $upcoming2 = factory(Event::class)->create([
            'name' => 'Upcoming event',
            'slug' => 'upcoming-event',
            'status' => 'open',
        ]);
        $upcoming2->agendas()->create([
            'date' => now()->addDay(),
        ]);

        $previous = factory(Event::class)->create([
            'name' => 'Event in the past',
            'slug' => 'event-in-the-past',
            'status' => 'open',
        ]);
        $previous->agendas()->create([
            'date' => now()->subDay(),
        ]);

        $upcoming->delegates()->save($delegate, ['status' => 'invited', 'pathway_id' => 1]);
        $upcoming2->delegates()->save($delegate, ['status' => 'invited', 'pathway_id' => 1]);
        $previous->delegates()->save($delegate, ['status' => 'attended', 'pathway_id' => 1]);

        $this->assertMatchesSnapshot(DelegateDashboard::make($delegate)->get());
    }

    /** @test */
    public function only_gets_a_max_of_3_upcoming_and_previous()
    {
        Carbon::setTestNow(Carbon::parse('08 September 2020'));
        config()->set('dashboards.delegate.show_max_upcoming', 3);
        config()->set('dashboards.delegate.show_max_previous', 3);

        $delegate = factory(Delegate::class)->create();

        $upcoming = factory(Event::class, 4)->create([
            'name' => 'Upcoming event',
            'slug' => 'upcoming-event',
            'status' => 'open',
        ]);
        $upcoming->each(static function (Event $event) use ($delegate): void {
            $event->delegates()->save($delegate, ['status' => 'invited', 'pathway_id' => 1]);
            $event->agendas()->create([
                'date' => now()->addDays($event->id),
            ]);
        });

        $previous = factory(Event::class, 4)->create([
            'name' => 'Event in the past',
            'slug' => 'event-in-the-past',
            'status' => 'open',
        ]);
        $previous->each(static function (Event $event) use ($delegate): void {
            $event->delegates()->save($delegate, ['status' => 'attended', 'pathway_id' => 1]);
            $event->agendas()->create([
                'date' => now()->subDays($event->id),
            ]);
        });

        $this->assertMatchesSnapshot(DelegateDashboard::make($delegate)->get());
    }

    /** @test */
    public function delegate_has_no_events()
    {
        $delegate = factory(Delegate::class)->create();

        $this->assertMatchesSnapshot(DelegateDashboard::make($delegate)->get());
    }
}
